@@ -1,71 +1,53 @@
 from __future__ import annotations
 
-from datetime import datetime
-from datetime import timedelta
+import datetime as dt
 from decimal import Decimal
 from functools import wraps
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Literal
-from typing import TypeVar
-from typing import overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, overload
 
-from narwhals._namespace import is_native_arrow
-from narwhals._namespace import is_native_pandas_like
-from narwhals._namespace import is_native_polars
-from narwhals._namespace import is_native_spark_like
-from narwhals.dependencies import get_cudf
-from narwhals.dependencies import get_cupy
-from narwhals.dependencies import get_dask
-from narwhals.dependencies import get_dask_expr
-from narwhals.dependencies import get_modin
-from narwhals.dependencies import get_numpy
-from narwhals.dependencies import get_pandas
-from narwhals.dependencies import get_polars
-from narwhals.dependencies import get_pyarrow
-from narwhals.dependencies import is_cudf_dataframe
-from narwhals.dependencies import is_cudf_series
-from narwhals.dependencies import is_dask_dataframe
-from narwhals.dependencies import is_duckdb_relation
-from narwhals.dependencies import is_ibis_table
-from narwhals.dependencies import is_modin_dataframe
-from narwhals.dependencies import is_modin_series
-from narwhals.dependencies import is_pandas_dataframe
-from narwhals.dependencies import is_pandas_like_dataframe
-from narwhals.dependencies import is_pandas_series
-from narwhals.dependencies import is_polars_dataframe
-from narwhals.dependencies import is_polars_lazyframe
-from narwhals.dependencies import is_polars_series
-from narwhals.dependencies import is_pyarrow_chunked_array
-from narwhals.dependencies import is_pyarrow_table
-from narwhals.utils import Version
+from narwhals._constants import EPOCH, MS_PER_SECOND
+from narwhals._namespace import (
+    is_native_arrow,
+    is_native_pandas_like,
+    is_native_polars,
+    is_native_spark_like,
+)
+from narwhals._utils import Implementation, Version, has_native_namespace
+from narwhals.dependencies import (
+    get_dask_expr,
+    get_numpy,
+    get_pandas,
+    is_cupy_scalar,
+    is_dask_dataframe,
+    is_duckdb_relation,
+    is_ibis_table,
+    is_numpy_scalar,
+    is_pandas_like_dataframe,
+    is_polars_lazyframe,
+    is_polars_series,
+    is_pyarrow_scalar,
+    is_pyarrow_table,
+)
 
 if TYPE_CHECKING:
-    from narwhals.dataframe import DataFrame
-    from narwhals.dataframe import LazyFrame
+    from narwhals.dataframe import DataFrame, LazyFrame
     from narwhals.series import Series
-    from narwhals.typing import DataFrameT
-    from narwhals.typing import IntoDataFrameT
-    from narwhals.typing import IntoFrame
-    from narwhals.typing import IntoFrameT
-    from narwhals.typing import IntoLazyFrameT
-    from narwhals.typing import IntoSeries
-    from narwhals.typing import IntoSeriesT
-    from narwhals.typing import LazyFrameT
-    from narwhals.typing import SeriesT
+    from narwhals.typing import (
+        DataFrameT,
+        Frame,
+        IntoDataFrameT,
+        IntoFrame,
+        IntoLazyFrameT,
+        IntoSeries,
+        IntoSeriesT,
+        LazyFrameT,
+        SeriesT,
+    )
 
 T = TypeVar("T")
 
-NON_TEMPORAL_SCALAR_TYPES = (
-    bool,
-    bytes,
-    str,
-    int,
-    float,
-    complex,
-    Decimal,
-)
+NON_TEMPORAL_SCALAR_TYPES = (bool, bytes, str, int, float, complex, Decimal)
+TEMPORAL_SCALAR_TYPES = (dt.date, dt.timedelta, dt.time)
 
 
 @overload
@@ -74,8 +56,8 @@ def to_native(
 ) -> IntoDataFrameT: ...
 @overload
 def to_native(
-    narwhals_object: LazyFrame[IntoFrameT], *, pass_through: Literal[False] = ...
-) -> IntoFrameT: ...
+    narwhals_object: LazyFrame[IntoLazyFrameT], *, pass_through: Literal[False] = ...
+) -> IntoLazyFrameT: ...
 @overload
 def to_native(
     narwhals_object: Series[IntoSeriesT], *, pass_through: Literal[False] = ...
@@ -86,26 +68,16 @@ def to_native(narwhals_object: Any, *, pass_through: bool) -> Any: ...
 
 def to_native(
     narwhals_object: DataFrame[IntoDataFrameT]
-    | LazyFrame[IntoFrameT]
+    | LazyFrame[IntoLazyFrameT]
     | Series[IntoSeriesT],
     *,
-    strict: bool | None = None,
-    pass_through: bool | None = None,
-) -> IntoDataFrameT | IntoFrameT | IntoSeriesT | Any:
+    pass_through: bool = False,
+) -> IntoDataFrameT | IntoLazyFrameT | IntoSeriesT | Any:
     """Convert Narwhals object to native one.
 
     Arguments:
         narwhals_object: Narwhals object.
-        strict: Determine what happens if `narwhals_object` isn't a Narwhals class:
-
-            - `True` (default): raise an error
-            - `False`: pass object through as-is
-
-            **Deprecated** (v1.13.0):
-                Please use `pass_through` instead. Note that `strict` is still available
-                (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-                see [perfect backwards compatibility policy](../backcompat.md/).
-        pass_through: Determine what happens if `narwhals_object` isn't a Narwhals class:
+        pass_through: Determine what happens if `narwhals_object` isn't a Narwhals class
 
             - `False` (default): raise an error
             - `True`: pass object through as-is
@@ -115,11 +87,6 @@ def to_native(
     """
     from narwhals.dataframe import BaseFrame
     from narwhals.series import Series
-    from narwhals.utils import validate_strict_and_pass_though
-
-    pass_through = validate_strict_and_pass_though(
-        strict, pass_through, pass_through_default=False, emit_deprecation_warning=True
-    )
 
     if isinstance(narwhals_object, BaseFrame):
         return narwhals_object._compliant_frame._native_frame
@@ -201,13 +168,13 @@ def from_native(
 
 @overload
 def from_native(
-    native_object: IntoFrameT | IntoLazyFrameT | IntoSeriesT,
+    native_object: IntoDataFrameT | IntoLazyFrameT | IntoSeriesT,
     *,
     pass_through: Literal[True],
     eager_only: Literal[False] = ...,
     series_only: Literal[False] = ...,
     allow_series: Literal[True],
-) -> DataFrame[IntoFrameT] | LazyFrame[IntoLazyFrameT] | Series[IntoSeriesT]: ...
+) -> DataFrame[IntoDataFrameT] | LazyFrame[IntoLazyFrameT] | Series[IntoSeriesT]: ...
 
 
 @overload
@@ -221,11 +188,8 @@ def from_native(
 ) -> Series[IntoSeriesT]: ...
 
 
-# NOTE: Seems like `mypy` is giving a false positive
-# Following this advice will introduce overlapping overloads?
-# > note: Flipping the order of overloads will fix this error
 @overload
-def from_native(  # type: ignore[overload-overlap]
+def from_native(
     native_object: IntoLazyFrameT,
     *,
     pass_through: Literal[False] = ...,
@@ -292,46 +256,41 @@ def from_native(
 
 
 def from_native(  # noqa: D417
-    native_object: IntoLazyFrameT | IntoFrameT | IntoSeriesT | IntoFrame | IntoSeries | T,
+    native_object: IntoLazyFrameT
+    | IntoDataFrameT
+    | IntoSeriesT
+    | IntoFrame
+    | IntoSeries
+    | T,
     *,
-    strict: bool | None = None,
-    pass_through: bool | None = None,
+    pass_through: bool = False,
     eager_only: bool = False,
     series_only: bool = False,
     allow_series: bool | None = None,
     **kwds: Any,
-) -> LazyFrame[IntoLazyFrameT] | DataFrame[IntoFrameT] | Series[IntoSeriesT] | T:
+) -> LazyFrame[IntoLazyFrameT] | DataFrame[IntoDataFrameT] | Series[IntoSeriesT] | T:
     """Convert `native_object` to Narwhals Dataframe, Lazyframe, or Series.
 
     Arguments:
         native_object: Raw object from user.
-            Depending on the other arguments, input object can be:
+            Depending on the other arguments, input object can be
 
             - a Dataframe / Lazyframe / Series supported by Narwhals (pandas, Polars, PyArrow, ...)
             - an object which implements `__narwhals_dataframe__`, `__narwhals_lazyframe__`,
               or `__narwhals_series__`
-        strict: Determine what happens if the object can't be converted to Narwhals:
+        pass_through: Determine what happens if the object can't be converted to Narwhals
 
-            - `True` or `None` (default): raise an error
-            - `False`: pass object through as-is
-
-            **Deprecated** (v1.13.0):
-                Please use `pass_through` instead. Note that `strict` is still available
-                (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-                see [perfect backwards compatibility policy](../backcompat.md/).
-        pass_through: Determine what happens if the object can't be converted to Narwhals:
-
-            - `False` or `None` (default): raise an error
+            - `False` (default): raise an error
             - `True`: pass object through as-is
-        eager_only: Whether to only allow eager objects:
+        eager_only: Whether to only allow eager objects
 
             - `False` (default): don't require `native_object` to be eager
             - `True`: only convert to Narwhals if `native_object` is eager
-        series_only: Whether to only allow Series:
+        series_only: Whether to only allow Series
 
             - `False` (default): don't require `native_object` to be a Series
             - `True`: only convert to Narwhals if `native_object` is a Series
-        allow_series: Whether to allow Series (default is only Dataframe / Lazyframe):
+        allow_series: Whether to allow Series (default is only Dataframe / Lazyframe)
 
             - `False` or `None` (default): don't convert to Narwhals if `native_object` is a Series
             - `True`: allow `native_object` to be a Series
@@ -340,11 +299,6 @@ def from_native(  # noqa: D417
         DataFrame, LazyFrame, Series, or original object, depending
             on which combination of parameters was passed.
     """
-    from narwhals.utils import validate_strict_and_pass_though
-
-    pass_through = validate_strict_and_pass_though(
-        strict, pass_through, pass_through_default=False, emit_deprecation_warning=True
-    )
     if kwds:
         msg = f"from_native() got an unexpected keyword argument {next(iter(kwds))!r}"
         raise TypeError(msg)
@@ -371,14 +325,14 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
     allow_series: bool | None = None,
     version: Version,
 ) -> Any:
-    from narwhals.dataframe import DataFrame
-    from narwhals.dataframe import LazyFrame
+    from narwhals._interchange.dataframe import supports_dataframe_interchange
+    from narwhals._utils import (
+        is_compliant_dataframe,
+        is_compliant_lazyframe,
+        is_compliant_series,
+    )
+    from narwhals.dataframe import DataFrame, LazyFrame
     from narwhals.series import Series
-    from narwhals.utils import _supports_dataframe_interchange
-    from narwhals.utils import is_compliant_dataframe
-    from narwhals.utils import is_compliant_lazyframe
-    from narwhals.utils import is_compliant_series
-    from narwhals.utils import parse_version
 
     # Early returns
     if isinstance(native_object, (DataFrame, LazyFrame)) and not series_only:
@@ -402,11 +356,10 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 msg = "Cannot only use `series_only` with dataframe"
                 raise TypeError(msg)
             return native_object
-        return DataFrame(
-            native_object.__narwhals_dataframe__(),
-            level="full",
+        return version.dataframe(
+            native_object.__narwhals_dataframe__()._with_version(version), level="full"
         )
-    elif is_compliant_lazyframe(native_object):
+    if is_compliant_lazyframe(native_object):
         if series_only:
             if not pass_through:
                 msg = "Cannot only use `series_only` with lazyframe"
@@ -417,23 +370,21 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 msg = "Cannot only use `eager_only` or `eager_or_interchange_only` with lazyframe"
                 raise TypeError(msg)
             return native_object
-        return LazyFrame(
-            native_object.__narwhals_lazyframe__(),
-            level="full",
+        return version.lazyframe(
+            native_object.__narwhals_lazyframe__()._with_version(version), level="full"
         )
-    elif is_compliant_series(native_object):
+    if is_compliant_series(native_object):
         if not allow_series:
             if not pass_through:
                 msg = "Please set `allow_series=True` or `series_only=True`"
                 raise TypeError(msg)
             return native_object
-        return Series(
-            native_object.__narwhals_series__(),
-            level="full",
+        return version.series(
+            native_object.__narwhals_series__()._with_version(version), level="full"
         )
 
     # Polars
-    elif is_native_polars(native_object):
+    if is_native_polars(native_object):
         if series_only and not is_polars_series(native_object):
             if not pass_through:
                 msg = f"Cannot only use `series_only` with {type(native_object).__qualname__}"
@@ -458,7 +409,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # PandasLike
-    elif is_native_pandas_like(native_object):
+    if is_native_pandas_like(native_object):
         if is_pandas_like_dataframe(native_object):
             if series_only:
                 if not pass_through:
@@ -477,7 +428,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # PyArrow
-    elif is_native_arrow(native_object):
+    if is_native_arrow(native_object):
         if is_pyarrow_table(native_object):
             if series_only:
                 if not pass_through:
@@ -496,9 +447,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # Dask
-    elif is_dask_dataframe(native_object):
-        from narwhals._dask.namespace import DaskNamespace
-
+    if is_dask_dataframe(native_object):
         if series_only:
             if not pass_through:
                 msg = "Cannot only use `series_only` with dask DataFrame"
@@ -509,18 +458,20 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 msg = "Cannot only use `eager_only` or `eager_or_interchange_only` with dask DataFrame"
                 raise TypeError(msg)
             return native_object
-        dask_version = parse_version(get_dask())
-        if dask_version <= (2024, 12, 1) and get_dask_expr() is None:  # pragma: no cover
+        if (
+            Implementation.DASK._backend_version() <= (2024, 12, 1)
+            and get_dask_expr() is None
+        ):  # pragma: no cover
             msg = "Please install dask-expr"
             raise ImportError(msg)
         return (
-            DaskNamespace(backend_version=dask_version, version=version)
-            .from_native(native_object)
+            version.namespace.from_backend(Implementation.DASK)
+            .compliant.from_native(native_object)
             .to_narwhals()
         )
 
     # DuckDB
-    elif is_duckdb_relation(native_object):
+    if is_duckdb_relation(native_object):
         if eager_only or series_only:  # pragma: no cover
             if not pass_through:
                 msg = "Cannot only use `series_only=True` or `eager_only=False` with DuckDBPyRelation"
@@ -533,49 +484,33 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # Ibis
-    elif is_ibis_table(native_object):  # pragma: no cover
-        from narwhals._ibis.dataframe import IbisLazyFrame
-
-        if eager_only or series_only:
+    if is_ibis_table(native_object):
+        if eager_only or series_only:  # pragma: no cover
             if not pass_through:
-                msg = (
-                    "Cannot only use `series_only=True` or `eager_only=False` "
-                    "with Ibis table"
-                )
+                msg = "Cannot only use `series_only=True` or `eager_only=False` with ibis.Table"
                 raise TypeError(msg)
             return native_object
-        import ibis  # ignore-banned-import
-
-        backend_version = parse_version(ibis)
-        if version is Version.V1:
-            return DataFrame(
-                IbisLazyFrame(
-                    native_object, backend_version=backend_version, version=version
-                ),
-                level="interchange",
-            )
-        return LazyFrame(
-            IbisLazyFrame(
-                native_object, backend_version=backend_version, version=version
-            ),
-            level="lazy",
+        return (
+            version.namespace.from_native_object(native_object)
+            .compliant.from_native(native_object)
+            .to_narwhals()
         )
 
     # PySpark
-    elif is_native_spark_like(native_object):  # pragma: no cover
+    if is_native_spark_like(native_object):  # pragma: no cover
         ns_spark = version.namespace.from_native_object(native_object)
-        if series_only:
-            msg = (
-                f"Cannot only use `series_only` with {ns_spark.implementation} DataFrame"
-            )
-            raise TypeError(msg)
-        if eager_only or eager_or_interchange_only:
-            msg = f"Cannot only use `eager_only` or `eager_or_interchange_only` with {ns_spark.implementation} DataFrame"
-            raise TypeError(msg)
+        if series_only or eager_only or eager_or_interchange_only:
+            if not pass_through:
+                msg = (
+                    "Cannot only use `series_only`, `eager_only` or `eager_or_interchange_only` "
+                    f"with {ns_spark.implementation} DataFrame"
+                )
+                raise TypeError(msg)
+            return native_object
         return ns_spark.compliant.from_native(native_object).to_narwhals()
 
     # Interchange protocol
-    elif _supports_dataframe_interchange(native_object):
+    if supports_dataframe_interchange(native_object):
         from narwhals._interchange.dataframe import InterchangeFrame
 
         if eager_only or series_only:
@@ -586,20 +521,26 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 )
                 raise TypeError(msg)
             return native_object
-        return DataFrame(
-            InterchangeFrame(native_object, version=version),
-            level="interchange",
-        )
+        if version is not Version.V1:
+            if pass_through:
+                return native_object
+            msg = (
+                "The Dataframe Interchange Protocol is no longer supported in the main `narwhals` namespace.\n\n"
+                "You may want to:\n"
+                " - Use `narwhals.stable.v1`, where it is still supported.\n"
+                "    - See https://narwhals-dev.github.io/narwhals/backcompat\n"
+                " - Use `pass_through=True` to pass the object through without raising."
+            )
+            raise TypeError(msg)
+        return Version.V1.dataframe(InterchangeFrame(native_object), level="interchange")
 
-    elif not pass_through:
+    if not pass_through:
         msg = f"Expected pandas-like dataframe, Polars dataframe, or Polars lazyframe, got: {type(native_object)}"
         raise TypeError(msg)
     return native_object
 
 
-def get_native_namespace(
-    *obj: DataFrame[Any] | LazyFrame[Any] | Series[Any] | IntoFrame | IntoSeries,
-) -> Any:
+def get_native_namespace(*obj: Frame | Series[Any] | IntoFrame | IntoSeries) -> Any:
     """Get native namespace from object.
 
     Arguments:
@@ -631,34 +572,20 @@ def get_native_namespace(
     return result.pop()
 
 
-def _get_native_namespace_single_obj(  # noqa: PLR0911
-    obj: DataFrame[Any] | LazyFrame[Any] | Series[Any] | IntoFrame | IntoSeries,
+def _get_native_namespace_single_obj(
+    obj: Frame | Series[Any] | IntoFrame | IntoSeries,
 ) -> Any:
-    from narwhals.utils import has_native_namespace
-
     if has_native_namespace(obj):
         return obj.__native_namespace__()
-    if is_pandas_dataframe(obj) or is_pandas_series(obj):
-        return get_pandas()
-    if is_modin_dataframe(obj) or is_modin_series(obj):  # pragma: no cover
-        return get_modin()
-    if is_pyarrow_table(obj) or is_pyarrow_chunked_array(obj):
-        return get_pyarrow()
-    if is_cudf_dataframe(obj) or is_cudf_series(obj):  # pragma: no cover
-        return get_cudf()
-    if is_dask_dataframe(obj):  # pragma: no cover
-        return get_dask()
-    if is_polars_dataframe(obj) or is_polars_lazyframe(obj) or is_polars_series(obj):
-        return get_polars()
-    msg = f"Could not get native namespace from object of type: {type(obj)}"
-    raise TypeError(msg)
+    return Version.MAIN.namespace.from_native_object(
+        obj
+    ).implementation.to_native_namespace()
 
 
 def narwhalify(
     func: Callable[..., Any] | None = None,
     *,
-    strict: bool | None = None,
-    pass_through: bool | None = None,
+    pass_through: bool = True,
     eager_only: bool = False,
     series_only: bool = False,
     allow_series: bool | None = True,
@@ -675,28 +602,19 @@ def narwhalify(
 
     Arguments:
         func: Function to wrap in a `from_native`-`to_native` block.
-        strict: **Deprecated** (v1.13.0):
-            Please use `pass_through` instead. Note that `strict` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
+        pass_through: Determine what happens if the object can't be converted to Narwhals
 
-            Determine what happens if the object can't be converted to Narwhals:
-
-            - `True` or `None` (default): raise an error
-            - `False`: pass object through as-is
-        pass_through: Determine what happens if the object can't be converted to Narwhals:
-
-            - `False` or `None` (default): raise an error
-            - `True`: pass object through as-is
-        eager_only: Whether to only allow eager objects:
+            - `False`: raise an error
+            - `True` (default): pass object through as-is
+        eager_only: Whether to only allow eager objects
 
             - `False` (default): don't require `native_object` to be eager
             - `True`: only convert to Narwhals if `native_object` is eager
-        series_only: Whether to only allow Series:
+        series_only: Whether to only allow Series
 
             - `False` (default): don't require `native_object` to be a Series
             - `True`: only convert to Narwhals if `native_object` is a Series
-        allow_series: Whether to allow Series (default is only Dataframe / Lazyframe):
+        allow_series: Whether to allow Series (default is only Dataframe / Lazyframe)
 
             - `False` or `None`: don't convert to Narwhals if `native_object` is a Series
             - `True` (default): allow `native_object` to be a Series
@@ -719,11 +637,6 @@ def narwhalify(
         ... def agnostic_group_by_sum(df):
         ...     return df.group_by("a").agg(nw.col("b").sum())
     """
-    from narwhals.utils import validate_strict_and_pass_though
-
-    pass_through = validate_strict_and_pass_though(
-        strict, pass_through, pass_through_default=True, emit_deprecation_warning=True
-    )
 
     def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
         @wraps(func)
@@ -768,12 +681,11 @@ def narwhalify(
 
     if func is None:
         return decorator
-    else:
-        # If func is not None, it means the decorator is used without arguments
-        return decorator(func)
+    # If func is not None, it means the decorator is used without arguments
+    return decorator(func)
 
 
-def to_py_scalar(scalar_like: Any) -> Any:  # noqa: C901, PLR0911, PLR0912
+def to_py_scalar(scalar_like: Any) -> Any:
     """If a scalar is not Python native, converts it to Python native.
 
     Arguments:
@@ -798,61 +710,42 @@ def to_py_scalar(scalar_like: Any) -> Any:  # noqa: C901, PLR0911, PLR0912
         >>> nw.to_py_scalar(1)
         1
     """
-    if scalar_like is None:
-        return None
-    if isinstance(scalar_like, NON_TEMPORAL_SCALAR_TYPES):
-        return scalar_like
-
-    np = get_numpy()
-    if (
-        np
+    scalar: Any
+    pd = get_pandas()
+    if scalar_like is None or isinstance(scalar_like, NON_TEMPORAL_SCALAR_TYPES):
+        scalar = scalar_like
+    elif (
+        (np := get_numpy())
         and isinstance(scalar_like, np.datetime64)
         and scalar_like.dtype == "datetime64[ns]"
     ):
-        return datetime(1970, 1, 1) + timedelta(microseconds=scalar_like.item() // 1000)
-
-    if np and np.isscalar(scalar_like) and hasattr(scalar_like, "item"):
-        return scalar_like.item()
-
-    pd = get_pandas()
-    if pd and isinstance(scalar_like, pd.Timestamp):
-        return scalar_like.to_pydatetime()
-    if pd and isinstance(scalar_like, pd.Timedelta):
-        return scalar_like.to_pytimedelta()
-    if pd and pd.api.types.is_scalar(scalar_like):
-        try:
-            is_na = pd.isna(scalar_like)
-        except Exception:  # pragma: no cover  # noqa: BLE001, S110
-            pass
-        else:
-            if is_na:
-                return None
-
+        ms = scalar_like.item() // MS_PER_SECOND
+        scalar = EPOCH + dt.timedelta(microseconds=ms)
+    elif is_numpy_scalar(scalar_like) or is_cupy_scalar(scalar_like):
+        scalar = scalar_like.item()
+    elif pd and isinstance(scalar_like, pd.Timestamp):
+        scalar = scalar_like.to_pydatetime()
+    elif pd and isinstance(scalar_like, pd.Timedelta):
+        scalar = scalar_like.to_pytimedelta()
     # pd.Timestamp and pd.Timedelta subclass datetime and timedelta,
     # so we need to check this separately
-    if isinstance(scalar_like, (datetime, timedelta)):
-        return scalar_like
-
-    pa = get_pyarrow()
-    if pa and isinstance(scalar_like, pa.Scalar):
-        return scalar_like.as_py()
-
-    cupy = get_cupy()
-    if (  # pragma: no cover
-        cupy and isinstance(scalar_like, cupy.ndarray) and scalar_like.size == 1
-    ):
-        return scalar_like.item()
-
-    msg = (
-        f"Expected object convertible to a scalar, found {type(scalar_like)}. "
-        "Please report a bug to https://github.com/narwhals-dev/narwhals/issues"
-    )
-    raise ValueError(msg)
+    elif isinstance(scalar_like, TEMPORAL_SCALAR_TYPES):
+        scalar = scalar_like
+    elif _is_pandas_na(scalar_like):
+        scalar = None
+    elif is_pyarrow_scalar(scalar_like):
+        scalar = scalar_like.as_py()
+    else:
+        msg = (
+            f"Expected object convertible to a scalar, found {type(scalar_like)}.\n"
+            f"{scalar_like!r}"
+        )
+        raise ValueError(msg)
+    return scalar
 
 
-__all__ = [
-    "get_native_namespace",
-    "narwhalify",
-    "to_native",
-    "to_py_scalar",
-]
+def _is_pandas_na(obj: Any) -> bool:
+    return bool((pd := get_pandas()) and pd.api.types.is_scalar(obj) and pd.isna(obj))
+
+
+__all__ = ["get_native_namespace", "narwhalify", "to_native", "to_py_scalar"]
